//----------------------------------------------------------------------------
//
// UMEBOSHI (http://en.wikipedia.org/wiki/Umeboshi)
//
//----------------------------------------------------------------------------
float4   cnt    : register(c0);

struct VS_OUTPUT {
  float4 pos : POSITION;
  float4 tex : TEXCOORD0;
};

VS_OUTPUT vs_main(VS_OUTPUT input) {
  return input;
}

float iso(float3 p) {
  float  x =    (p.x);
  float  y = cos(p.y) * 3;
  float  z =    (p.z) * 0.2;
  float3 ip = float3(34, 33, 32);
  float r = (cos(x) + cos(y) + cos(z));
  r -= log( sin(p.x) + sin(p.y) + sin(p.z * 2)) * 0.5;
  r -= log( sin(p.x) + sin(p.y * 2) + sin(p.z * 1.5)) * 0.02;
  p *= ip;
  r -=    ( sin(p.x) + sin(p.y) + sin(p.z)) * 0.05;
  return r;
}

float raycast(float3 eye, float3 dir, float th) {
  float dt = 7;
  float t = 0;
  for(int i = 0; i < 150 && dt > 0.001; i++) {
    float3 temp = eye + dir * t;
    if(iso(temp) < th) {
      t -= dt;
      dt *= 0.1;
    }
    t += dt;
  }
  return t;
}


float4 ps_main(float2 vp : vpos) : color {
  float2 uv = -1 + 2 * vp / (float2(800, 600));
  float time  = cnt.x;
  float flash = 0.1;//exp(1 - frac(time));
  
  if(uv.y < -0.7 || uv.y > 0.7) return 0;
  float rotate = (time + sin(time * 0.3)) * 0.1;
  float3x3 mtx = {
    cos(rotate),  sin(rotate), 0,
    sin(rotate), -cos(rotate), 0,
    0,           0,            1
  };
  
  //rotate ray direction
  float3 dir    = normalize(
    float3(uv.x * 1.25 - sin(time * 3) * 0.02,
           uv.y        + cos(time * 4) * 0.02,
           1));
  dir           = mul(mtx, dir).zxy;
  dir           = mul(mtx, dir) / 64;
  
  //cast
  float3 eye    = float3(0, time * 2 + flash * 5, 0);
  float  t      = raycast(eye, dir,  -0.7);
  
  //calc normal.
  float3 ip     = eye + dir * t;
  float  iit    = iso(ip);
  float3 h      = float3(0.01, 0, 0);
  float3 n      = normalize(float3(
    iit - iso(ip + h.xyy),
    iit - iso(ip + h.yxy),
    iit - iso(ip + h.yyx)));
  
  //Light
  float3 light  = normalize(float3(1,  -0.5,  1));
  float3 normal = dot(light, n);
  
  //iro
  float  len    = distance(eye + dir * t, eye);
  float4 col    = float4((dir * 0.5) + float3(0.9, 0.1, 0.0) * normal, 1) + t * 0.0005;
  return col + flash * 0.2;
  
}
